@@ -4,8 +4,9 @@ serialization via the pickle module.
 """
 import py
 import pytest
+import _pytest._code
 
-pythonlist = ['python2.6', 'python2.7', 'python3.3']
+pythonlist = ['python2.6', 'python2.7', 'python3.4', 'python3.5']
 @pytest.fixture(params=pythonlist)
 def python1(request, tmpdir):
     picklefile = tmpdir.join("data.pickle")
@@ -23,7 +24,7 @@ class Python:
         self.picklefile = picklefile
     def dumps(self, obj):
         dumpfile = self.picklefile.dirpath("dump.py")
-        dumpfile.write(py.code.Source("""
+        dumpfile.write(_pytest._code.Source("""
             import pickle
             f = open(%r, 'wb')
             s = pickle.dump(%r, f, protocol=2)
@@ -33,7 +34,7 @@ class Python:
 
     def load_and_is_true(self, expression):
         loadfile = self.picklefile.dirpath("load.py")
-        loadfile.write(py.code.Source("""
+        loadfile.write(_pytest._code.Source("""
             import pickle
             f = open(%r, 'rb')
             obj = pickle.load(f)
