@@ -1,9 +1,14 @@
 # -*- coding: utf-8 -*-
 
+import operator
 import _pytest
 import py
 import pytest
-from _pytest._code.code import ExceptionInfo, FormattedExcinfo, ReprExceptionInfo
+from _pytest._code.code import (
+    ExceptionInfo,
+    FormattedExcinfo,
+    ReprExceptionInfo,
+    ExceptionChainRepr)
 
 queue = py.builtin._tryimport('queue', 'Queue')
 
@@ -21,14 +26,23 @@ import pytest
 pytest_version_info = tuple(map(int, pytest.__version__.split(".")[:3]))
 
 class TWMock:
+    WRITE = object()
+
     def __init__(self):
         self.lines = []
+        self.is_writing = False
     def sep(self, sep, line=None):
         self.lines.append((sep, line))
+    def write(self, msg, **kw):
+        self.lines.append((TWMock.WRITE, msg))
     def line(self, line, **kw):
         self.lines.append(line)
     def markup(self, text, **kw):
         return text
+    def get_write_msg(self, idx):
+        flag, msg = self.lines[idx]
+        assert flag == TWMock.WRITE
+        return msg
 
     fullwidth = 80
 
@@ -143,6 +157,39 @@ class TestTraceback_f_g_h:
         ntraceback = traceback.filter()
         assert len(ntraceback) == len(traceback) - 1
 
+    @pytest.mark.parametrize('tracebackhide, matching', [
+        (lambda info: True, True),
+        (lambda info: False, False),
+        (operator.methodcaller('errisinstance', ValueError), True),
+        (operator.methodcaller('errisinstance', IndexError), False),
+    ])
+    def test_traceback_filter_selective(self, tracebackhide, matching):
+        def f():
+            #
+            raise ValueError
+            #
+        def g():
+            #
+            __tracebackhide__ = tracebackhide
+            f()
+            #
+        def h():
+            #
+            g()
+            #
+
+        excinfo = pytest.raises(ValueError, h)
+        traceback = excinfo.traceback
+        ntraceback = traceback.filter()
+        print('old: {0!r}'.format(traceback))
+        print('new: {0!r}'.format(ntraceback))
+
+        if matching:
+            assert len(ntraceback) == len(traceback) - 2
+        else:
+            # -1 because of the __tracebackhide__ in pytest.raises
+            assert len(ntraceback) == len(traceback) - 1
+
     def test_traceback_recursion_index(self):
         def f(n):
             if n < 10:
@@ -236,18 +283,6 @@ class TestTraceback_f_g_h:
         assert entry.lineno == co.firstlineno + 2
         assert entry.frame.code.name == 'g'
 
-def hello(x):
-    x + 5
-
-def test_tbentry_reinterpret():
-    try:
-        hello("hello")
-    except TypeError:
-        excinfo = _pytest._code.ExceptionInfo()
-    tbentry = excinfo.traceback[-1]
-    msg = tbentry.reinterpret()
-    assert msg.startswith("TypeError: ('hello' + 5)")
-
 def test_excinfo_exconly():
     excinfo = pytest.raises(ValueError, h)
     assert excinfo.exconly().startswith('ValueError')
@@ -323,11 +358,32 @@ def test_codepath_Queue_example():
     assert path.basename.lower() == "queue.py"
     assert path.check()
 
+def test_match_succeeds():
+    with pytest.raises(ZeroDivisionError) as excinfo:
+        0 / 0
+    excinfo.match(r'.*zero.*')
+
+def test_match_raises_error(testdir):
+    testdir.makepyfile("""
+        import pytest
+        def test_division_zero():
+            with pytest.raises(ZeroDivisionError) as excinfo:
+                0 / 0
+            excinfo.match(r'[123]+')
+    """)
+    result = testdir.runpytest()
+    assert result.ret != 0
+    result.stdout.fnmatch_lines([
+        "*AssertionError*Pattern*[123]*not found*",
+    ])
+
 class TestFormattedExcinfo:
-    def pytest_funcarg__importasmod(self, request):
+
+    @pytest.fixture
+    def importasmod(self, request):
         def importasmod(source):
             source = _pytest._code.Source(source)
-            tmpdir = request.getfuncargvalue("tmpdir")
+            tmpdir = request.getfixturevalue("tmpdir")
             modpath = tmpdir.join("mod.py")
             tmpdir.ensure("__init__.py")
             modpath.write(source)
@@ -372,7 +428,7 @@ class TestFormattedExcinfo:
         assert lines == [
             '    def f():',
             '>       assert 0',
-            'E       assert 0'
+            'E       AssertionError'
         ]
 
 
@@ -385,6 +441,8 @@ class TestFormattedExcinfo:
             excinfo = _pytest._code.ExceptionInfo()
         repr = pr.repr_excinfo(excinfo)
         assert repr.reprtraceback.reprentries[1].lines[0] == ">   ???"
+        if py.std.sys.version_info[0] >= 3:
+            assert repr.chain[0][0].reprentries[1].lines[0] == ">   ???"
 
     def test_repr_many_line_source_not_existing(self):
         pr = FormattedExcinfo()
@@ -398,6 +456,8 @@ raise ValueError()
             excinfo = _pytest._code.ExceptionInfo()
         repr = pr.repr_excinfo(excinfo)
         assert repr.reprtraceback.reprentries[1].lines[0] == ">   ???"
+        if py.std.sys.version_info[0] >= 3:
+            assert repr.chain[0][0].reprentries[1].lines[0] == ">   ???"
 
     def test_repr_source_failing_fullsource(self):
         pr = FormattedExcinfo()
@@ -418,7 +478,7 @@ raise ValueError()
             f_globals = {}
 
         class FakeTracebackEntry(_pytest._code.Traceback.Entry):
-            def __init__(self, tb):
+            def __init__(self, tb, excinfo=None):
                 self.lineno = 5+3
 
             @property
@@ -430,6 +490,7 @@ raise ValueError()
 
         class FakeExcinfo(_pytest._code.ExceptionInfo):
             typename = "Foo"
+            value = Exception()
             def __init__(self):
                 pass
 
@@ -447,10 +508,15 @@ raise ValueError()
         fail = IOError()  # noqa
         repr = pr.repr_excinfo(excinfo)
         assert repr.reprtraceback.reprentries[0].lines[0] == ">   ???"
+        if py.std.sys.version_info[0] >= 3:
+            assert repr.chain[0][0].reprentries[0].lines[0] == ">   ???"
+
 
         fail = py.error.ENOENT  # noqa
         repr = pr.repr_excinfo(excinfo)
         assert repr.reprtraceback.reprentries[0].lines[0] == ">   ???"
+        if py.std.sys.version_info[0] >= 3:
+            assert repr.chain[0][0].reprentries[0].lines[0] == ">   ???"
 
 
     def test_repr_local(self):
@@ -637,6 +703,9 @@ raise ValueError()
             repr = p.repr_excinfo(excinfo)
             assert repr.reprtraceback
             assert len(repr.reprtraceback.reprentries) == len(reprtb.reprentries)
+            if py.std.sys.version_info[0] >= 3:
+                assert repr.chain[0][0]
+                assert len(repr.chain[0][0].reprentries) == len(reprtb.reprentries)
             assert repr.reprcrash.path.endswith("mod.py")
             assert repr.reprcrash.message == "ValueError: 0"
 
@@ -698,23 +767,6 @@ raise ValueError()
             assert reprtb.extraline == "!!! Recursion detected (same locals & position)"
             assert str(reprtb)
 
-    def test_tb_entry_AssertionError(self, importasmod):
-        # probably this test is a bit redundant
-        # as py/magic/testing/test_assertion.py
-        # already tests correctness of
-        # assertion-reinterpretation  logic
-        mod = importasmod("""
-            def somefunc():
-                x = 1
-                assert x == 2
-        """)
-        excinfo = pytest.raises(AssertionError, mod.somefunc)
-
-        p = FormattedExcinfo()
-        reprentry = p.repr_traceback_entry(excinfo.traceback[-1], excinfo)
-        lines = reprentry.lines
-        assert lines[-1] == "E       assert 1 == 2"
-
     def test_reprexcinfo_getrepr(self, importasmod):
         mod = importasmod("""
             def f(x):
@@ -727,8 +779,13 @@ raise ValueError()
         for style in ("short", "long", "no"):
             for showlocals in (True, False):
                 repr = excinfo.getrepr(style=style, showlocals=showlocals)
-                assert isinstance(repr, ReprExceptionInfo)
+                if py.std.sys.version_info[0] < 3:
+                    assert isinstance(repr, ReprExceptionInfo)
                 assert repr.reprtraceback.style == style
+                if py.std.sys.version_info[0] >= 3:
+                    assert isinstance(repr, ExceptionChainRepr)
+                    for repr in repr.chain:
+                        assert repr[0].style == style
 
     def test_reprexcinfo_unicode(self):
         from _pytest._code.code import TerminalRepr
@@ -755,14 +812,18 @@ raise ValueError()
         assert tw.lines[0] == "    def f():"
         assert tw.lines[1] == ">       g(3)"
         assert tw.lines[2] == ""
-        assert tw.lines[3].endswith("mod.py:5: ")
-        assert tw.lines[4] == ("_ ", None)
-        assert tw.lines[5] == ""
-        assert tw.lines[6] == "    def g(x):"
-        assert tw.lines[7] == ">       raise ValueError(x)"
-        assert tw.lines[8] == "E       ValueError: 3"
-        assert tw.lines[9] == ""
-        assert tw.lines[10].endswith("mod.py:3: ValueError")
+        line = tw.get_write_msg(3)
+        assert line.endswith("mod.py")
+        assert tw.lines[4] == (":5: ")
+        assert tw.lines[5] == ("_ ", None)
+        assert tw.lines[6] == ""
+        assert tw.lines[7] == "    def g(x):"
+        assert tw.lines[8] == ">       raise ValueError(x)"
+        assert tw.lines[9] == "E       ValueError: 3"
+        assert tw.lines[10] == ""
+        line = tw.get_write_msg(11)
+        assert line.endswith("mod.py")
+        assert tw.lines[12] == ":3: ValueError"
 
     def test_toterminal_long_missing_source(self, importasmod, tmpdir):
         mod = importasmod("""
@@ -781,13 +842,17 @@ raise ValueError()
         tw.lines.pop(0)
         assert tw.lines[0] == ">   ???"
         assert tw.lines[1] == ""
-        assert tw.lines[2].endswith("mod.py:5: ")
-        assert tw.lines[3] == ("_ ", None)
-        assert tw.lines[4] == ""
-        assert tw.lines[5] == ">   ???"
-        assert tw.lines[6] == "E   ValueError: 3"
-        assert tw.lines[7] == ""
-        assert tw.lines[8].endswith("mod.py:3: ValueError")
+        line = tw.get_write_msg(2)
+        assert line.endswith("mod.py")
+        assert tw.lines[3] == ":5: "
+        assert tw.lines[4] == ("_ ", None)
+        assert tw.lines[5] == ""
+        assert tw.lines[6] == ">   ???"
+        assert tw.lines[7] == "E   ValueError: 3"
+        assert tw.lines[8] == ""
+        line = tw.get_write_msg(9)
+        assert line.endswith("mod.py")
+        assert tw.lines[10] == ":3: ValueError"
 
     def test_toterminal_long_incomplete_source(self, importasmod, tmpdir):
         mod = importasmod("""
@@ -806,13 +871,17 @@ raise ValueError()
         tw.lines.pop(0)
         assert tw.lines[0] == ">   ???"
         assert tw.lines[1] == ""
-        assert tw.lines[2].endswith("mod.py:5: ")
-        assert tw.lines[3] == ("_ ", None)
-        assert tw.lines[4] == ""
-        assert tw.lines[5] == ">   ???"
-        assert tw.lines[6] == "E   ValueError: 3"
-        assert tw.lines[7] == ""
-        assert tw.lines[8].endswith("mod.py:3: ValueError")
+        line = tw.get_write_msg(2)
+        assert line.endswith("mod.py")
+        assert tw.lines[3] == ":5: "
+        assert tw.lines[4] == ("_ ", None)
+        assert tw.lines[5] == ""
+        assert tw.lines[6] == ">   ???"
+        assert tw.lines[7] == "E   ValueError: 3"
+        assert tw.lines[8] == ""
+        line = tw.get_write_msg(9)
+        assert line.endswith("mod.py")
+        assert tw.lines[10] == ":3: ValueError"
 
     def test_toterminal_long_filenames(self, importasmod):
         mod = importasmod("""
@@ -826,15 +895,18 @@ raise ValueError()
         try:
             repr = excinfo.getrepr(abspath=False)
             repr.toterminal(tw)
-            line = tw.lines[-1]
             x = py.path.local().bestrelpath(path)
             if len(x) < len(str(path)):
-                assert line == "mod.py:3: ValueError"
+                msg = tw.get_write_msg(-2)
+                assert msg == "mod.py"
+                assert tw.lines[-1] == ":3: ValueError"
 
             repr = excinfo.getrepr(abspath=True)
             repr.toterminal(tw)
+            msg = tw.get_write_msg(-2)
+            assert msg == path
             line = tw.lines[-1]
-            assert line == "%s:3: ValueError" %(path,)
+            assert line == ":3: ValueError"
         finally:
             old.chdir()
 
@@ -858,21 +930,6 @@ raise ValueError()
         repr.toterminal(tw)
         assert tw.stringio.getvalue()
 
-
-    def test_native_style(self):
-        excinfo = self.excinfo_from_exec("""
-            assert 0
-        """)
-        repr = excinfo.getrepr(style='native')
-        assert "assert 0" in str(repr.reprcrash)
-        s = str(repr)
-        assert s.startswith('Traceback (most recent call last):\n  File')
-        assert s.endswith('\nAssertionError: assert 0')
-        assert 'exec (source.compile())' in s
-        # python 2.4 fails to get the source line for the assert
-        if py.std.sys.version_info >= (2, 5):
-            assert s.count('assert 0') == 2
-
     def test_traceback_repr_style(self, importasmod):
         mod = importasmod("""
             def f():
@@ -896,19 +953,102 @@ raise ValueError()
         assert tw.lines[1] == "    def f():"
         assert tw.lines[2] == ">       g()"
         assert tw.lines[3] == ""
-        assert tw.lines[4].endswith("mod.py:3: ")
-        assert tw.lines[5] == ("_ ", None)
-        assert tw.lines[6].endswith("in g")
-        assert tw.lines[7] == "    h()"
-        assert tw.lines[8].endswith("in h")
-        assert tw.lines[9] == "    i()"
-        assert tw.lines[10] == ("_ ", None)
-        assert tw.lines[11] == ""
-        assert tw.lines[12] == "    def i():"
-        assert tw.lines[13] == ">       raise ValueError()"
-        assert tw.lines[14] == "E       ValueError"
+        msg = tw.get_write_msg(4)
+        assert msg.endswith("mod.py")
+        assert tw.lines[5] == ":3: "
+        assert tw.lines[6] == ("_ ", None)
+        tw.get_write_msg(7)
+        assert tw.lines[8].endswith("in g")
+        assert tw.lines[9] == "    h()"
+        tw.get_write_msg(10)
+        assert tw.lines[11].endswith("in h")
+        assert tw.lines[12] == "    i()"
+        assert tw.lines[13] == ("_ ", None)
+        assert tw.lines[14] == ""
+        assert tw.lines[15] == "    def i():"
+        assert tw.lines[16] == ">       raise ValueError()"
+        assert tw.lines[17] == "E       ValueError"
+        assert tw.lines[18] == ""
+        msg = tw.get_write_msg(19)
+        msg.endswith("mod.py")
+        assert tw.lines[20] == ":9: ValueError"
+
+    @pytest.mark.skipif("sys.version_info[0] < 3")
+    def test_exc_chain_repr(self, importasmod):
+        mod = importasmod("""
+            class Err(Exception):
+                pass
+            def f():
+                try:
+                    g()
+                except Exception as e:
+                    raise Err() from e
+                finally:
+                    h()
+            def g():
+                raise ValueError()
+
+            def h():
+                raise AttributeError()
+        """)
+        excinfo = pytest.raises(AttributeError, mod.f)
+        r = excinfo.getrepr(style="long")
+        tw = TWMock()
+        r.toterminal(tw)
+        for line in tw.lines: print (line)
+        assert tw.lines[0]  == ""
+        assert tw.lines[1]  == "    def f():"
+        assert tw.lines[2]  == "        try:"
+        assert tw.lines[3]  == ">           g()"
+        assert tw.lines[4]  == ""
+        line = tw.get_write_msg(5)
+        assert line.endswith('mod.py')
+        assert tw.lines[6]  == ':6: '
+        assert tw.lines[7]  == ("_ ", None)
+        assert tw.lines[8]  == ""
+        assert tw.lines[9]  == "    def g():"
+        assert tw.lines[10]  == ">       raise ValueError()"
+        assert tw.lines[11] == "E       ValueError"
+        assert tw.lines[12] == ""
+        line = tw.get_write_msg(13)
+        assert line.endswith('mod.py')
+        assert tw.lines[14] == ':12: ValueError'
         assert tw.lines[15] == ""
-        assert tw.lines[16].endswith("mod.py:9: ValueError")
+        assert tw.lines[16] == "The above exception was the direct cause of the following exception:"
+        assert tw.lines[17] == ""
+        assert tw.lines[18] == "    def f():"
+        assert tw.lines[19] == "        try:"
+        assert tw.lines[20] == "            g()"
+        assert tw.lines[21] == "        except Exception as e:"
+        assert tw.lines[22] == ">           raise Err() from e"
+        assert tw.lines[23] == "E           test_exc_chain_repr0.mod.Err"
+        assert tw.lines[24] == ""
+        line = tw.get_write_msg(25)
+        assert line.endswith('mod.py')
+        assert tw.lines[26] == ":8: Err"
+        assert tw.lines[27] == ""
+        assert tw.lines[28] == "During handling of the above exception, another exception occurred:"
+        assert tw.lines[29] == ""
+        assert tw.lines[30] == "    def f():"
+        assert tw.lines[31] == "        try:"
+        assert tw.lines[32] == "            g()"
+        assert tw.lines[33] == "        except Exception as e:"
+        assert tw.lines[34] == "            raise Err() from e"
+        assert tw.lines[35] == "        finally:"
+        assert tw.lines[36] == ">           h()"
+        assert tw.lines[37] == ""
+        line = tw.get_write_msg(38)
+        assert line.endswith('mod.py')
+        assert tw.lines[39] == ":10: "
+        assert tw.lines[40] == ('_ ', None)
+        assert tw.lines[41] == ""
+        assert tw.lines[42] == "    def h():"
+        assert tw.lines[43] == ">       raise AttributeError()"
+        assert tw.lines[44] == "E       AttributeError"
+        assert tw.lines[45] == ""
+        line = tw.get_write_msg(46)
+        assert line.endswith('mod.py')
+        assert tw.lines[47] == ":15: AttributeError"
 
 
 @pytest.mark.parametrize("style", ["short", "long"])
@@ -925,3 +1065,14 @@ def test_repr_traceback_with_unicode(style, encoding):
     repr_traceback = formatter.repr_traceback(e_info)
     assert repr_traceback is not None
 
+
+def test_cwd_deleted(testdir):
+    testdir.makepyfile("""
+        def test(tmpdir):
+            tmpdir.chdir()
+            tmpdir.remove()
+            assert False
+    """)
+    result = testdir.runpytest()
+    result.stdout.fnmatch_lines(['* 1 failed in *'])
+    assert 'INTERNALERROR' not in result.stdout.str() + result.stderr.str()
