@@ -4,6 +4,8 @@ from __future__ import with_statement
 import pickle
 import os
 import sys
+
+import _pytest._code
 import py
 import pytest
 import contextlib
@@ -414,11 +416,30 @@ class TestCaptureFixture:
         result = testdir.runpytest(p)
         result.stdout.fnmatch_lines([
             "*ERROR*setup*test_one*",
-            "*capsys*capfd*same*time*",
+            "E*capsys*capfd*same*time*",
             "*ERROR*setup*test_two*",
-            "*capsys*capfd*same*time*",
+            "E*capsys*capfd*same*time*",
             "*2 error*"])
 
+    def test_capturing_getfixturevalue(self, testdir):
+        """Test that asking for "capfd" and "capsys" using request.getfixturevalue
+        in the same test is an error.
+        """
+        testdir.makepyfile("""
+            def test_one(capsys, request):
+                request.getfixturevalue("capfd")
+            def test_two(capfd, request):
+                request.getfixturevalue("capsys")
+        """)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines([
+            "*test_one*",
+            "*capsys*capfd*same*time*",
+            "*test_two*",
+            "*capsys*capfd*same*time*",
+            "*2 failed in*",
+        ])
+
     @pytest.mark.parametrize("method", ["sys", "fd"])
     def test_capture_is_represented_on_failure_issue128(self, testdir, method):
         p = testdir.makepyfile("""
@@ -478,10 +499,26 @@ class TestCaptureFixture:
         result = testdir.runpytest_subprocess(p)
         assert 'closed' not in result.stderr.str()
 
+    @pytest.mark.parametrize('fixture', ['capsys', 'capfd'])
+    def test_disabled_capture_fixture(self, testdir, fixture):
+        testdir.makepyfile("""
+            def test_disabled({fixture}):
+                print('captured before')
+                with {fixture}.disabled():
+                    print('while capture is disabled')
+                print('captured after')
+        """.format(fixture=fixture))
+        result = testdir.runpytest_subprocess()
+        result.stdout.fnmatch_lines("""
+            *while capture is disabled*
+        """)
+        assert 'captured before' not in result.stdout.str()
+        assert 'captured after' not in result.stdout.str()
+
 
 def test_setup_failure_does_not_kill_capturing(testdir):
     sub1 = testdir.mkpydir("sub1")
-    sub1.join("conftest.py").write(py.code.Source("""
+    sub1.join("conftest.py").write(_pytest._code.Source("""
         def pytest_runtest_setup(item):
             raise ValueError(42)
     """))
@@ -556,7 +593,6 @@ def test_capture_binary_output(testdir):
             import subprocess
             subprocess.call([sys.executable, __file__])
 
-        @pytest.mark.skip
         def test_foo():
             import os;os.write(1, b'\xc3')
 
@@ -626,6 +662,28 @@ def test_dontreadfrominput():
     f.close()  # just for completeness
 
 
+@pytest.mark.skipif('sys.version_info < (3,)', reason='python2 has no buffer')
+def test_dontreadfrominput_buffer_python3():
+    from _pytest.capture import DontReadFromInput
+    f = DontReadFromInput()
+    fb = f.buffer
+    assert not fb.isatty()
+    pytest.raises(IOError, fb.read)
+    pytest.raises(IOError, fb.readlines)
+    pytest.raises(IOError, iter, fb)
+    pytest.raises(ValueError, fb.fileno)
+    f.close()  # just for completeness
+
+
+@pytest.mark.skipif('sys.version_info >= (3,)', reason='python2 has no buffer')
+def test_dontreadfrominput_buffer_python2():
+    from _pytest.capture import DontReadFromInput
+    f = DontReadFromInput()
+    with pytest.raises(AttributeError):
+        f.buffer
+    f.close()  # just for completeness
+
+
 @pytest.yield_fixture
 def tmpfile(testdir):
     f = testdir.makepyfile("").open('wb+')
