@@ -1,7 +1,7 @@
 # encoding: utf-8
 import sys
+import _pytest._code
 from _pytest.doctest import DoctestItem, DoctestModule, DoctestTextfile
-import py
 import pytest
 
 class TestDoctests:
@@ -14,13 +14,16 @@ class TestDoctests:
             >>> i-1
             4
         """)
+
         for x in (testdir.tmpdir, checkfile):
             #print "checking that %s returns custom items" % (x,)
             items, reprec = testdir.inline_genitems(x)
             assert len(items) == 1
-            assert isinstance(items[0], DoctestTextfile)
+            assert isinstance(items[0], DoctestItem)
+            assert isinstance(items[0].parent, DoctestTextfile)
+        # Empty file has no items.
         items, reprec = testdir.inline_genitems(w)
-        assert len(items) == 1
+        assert len(items) == 0
 
     def test_collect_module_empty(self, testdir):
         path = testdir.makepyfile(whatever="#")
@@ -88,7 +91,7 @@ class TestDoctests:
         reprec.assertoutcome(failed=1)
 
     def test_new_pattern(self, testdir):
-        p = testdir.maketxtfile(xdoc ="""
+        p = testdir.maketxtfile(xdoc="""
             >>> x = 1
             >>> x == 1
             False
@@ -96,6 +99,36 @@ class TestDoctests:
         reprec = testdir.inline_run(p, "--doctest-glob=x*.txt")
         reprec.assertoutcome(failed=1)
 
+    def test_multiple_patterns(self, testdir):
+        """Test support for multiple --doctest-glob arguments (#1255).
+        """
+        testdir.maketxtfile(xdoc="""
+            >>> 1
+            1
+        """)
+        testdir.makefile('.foo', test="""
+            >>> 1
+            1
+        """)
+        testdir.maketxtfile(test_normal="""
+            >>> 1
+            1
+        """)
+        expected = set(['xdoc.txt', 'test.foo', 'test_normal.txt'])
+        assert set(x.basename for x in testdir.tmpdir.listdir()) == expected
+        args = ["--doctest-glob=xdoc*.txt", "--doctest-glob=*.foo"]
+        result = testdir.runpytest(*args)
+        result.stdout.fnmatch_lines([
+            '*test.foo *',
+            '*xdoc.txt *',
+            '*2 passed*',
+        ])
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines([
+            '*test_normal.txt *',
+            '*1 passed*',
+        ])
+
     def test_doctest_unexpected_exception(self, testdir):
         testdir.maketxtfile("""
             >>> i = 0
@@ -151,7 +184,7 @@ class TestDoctests:
         assert 'text-line-after' not in result.stdout.str()
 
     def test_doctest_linedata_missing(self, testdir):
-        testdir.tmpdir.join('hello.py').write(py.code.Source("""
+        testdir.tmpdir.join('hello.py').write(_pytest._code.Source("""
             class Fun(object):
                 @property
                 def test(self):
@@ -169,9 +202,21 @@ class TestDoctests:
             "*1 failed*",
         ])
 
+    def test_doctest_unex_importerror_only_txt(self, testdir):
+        testdir.maketxtfile("""
+            >>> import asdalsdkjaslkdjasd
+            >>>
+        """)
+        result = testdir.runpytest()
+        # doctest is never executed because of error during hello.py collection
+        result.stdout.fnmatch_lines([
+            "*>>> import asdals*",
+            "*UNEXPECTED*ImportError*",
+            "ImportError: No module named *asdal*",
+        ])
 
-    def test_doctest_unex_importerror(self, testdir):
-        testdir.tmpdir.join("hello.py").write(py.code.Source("""
+    def test_doctest_unex_importerror_with_module(self, testdir):
+        testdir.tmpdir.join("hello.py").write(_pytest._code.Source("""
             import asdalsdkjaslkdjasd
         """))
         testdir.maketxtfile("""
@@ -179,10 +224,11 @@ class TestDoctests:
             >>>
         """)
         result = testdir.runpytest("--doctest-modules")
+        # doctest is never executed because of error during hello.py collection
         result.stdout.fnmatch_lines([
-            "*>>> import hello",
-            "*UNEXPECTED*ImportError*",
-            "*import asdals*",
+            "*ERROR collecting hello.py*",
+            "*ImportError: No module named *asdals*",
+            "*Interrupted: 1 errors during collection*",
         ])
 
     def test_doctestmodule(self, testdir):
@@ -199,7 +245,7 @@ class TestDoctests:
 
     def test_doctestmodule_external_and_issue116(self, testdir):
         p = testdir.mkpydir("hello")
-        p.join("__init__.py").write(py.code.Source("""
+        p.join("__init__.py").write(_pytest._code.Source("""
             def somefunc():
                 '''
                     >>> i = 0
@@ -429,6 +475,9 @@ class TestDoctests:
                                     "--junit-xml=junit.xml")
         reprec.assertoutcome(failed=1)
 
+
+class TestLiterals:
+
     @pytest.mark.parametrize('config_mode', ['ini', 'comment'])
     def test_allow_unicode(self, testdir, config_mode):
         """Test that doctests which output unicode work in all python versions
@@ -458,6 +507,35 @@ class TestDoctests:
         reprec = testdir.inline_run("--doctest-modules")
         reprec.assertoutcome(passed=2)
 
+    @pytest.mark.parametrize('config_mode', ['ini', 'comment'])
+    def test_allow_bytes(self, testdir, config_mode):
+        """Test that doctests which output bytes work in all python versions
+        tested by pytest when the ALLOW_BYTES option is used (either in
+        the ini file or by an inline comment)(#1287).
+        """
+        if config_mode == 'ini':
+            testdir.makeini('''
+            [pytest]
+            doctest_optionflags = ALLOW_BYTES
+            ''')
+            comment = ''
+        else:
+            comment = '#doctest: +ALLOW_BYTES'
+
+        testdir.maketxtfile(test_doc="""
+            >>> b'foo'  {comment}
+            'foo'
+        """.format(comment=comment))
+        testdir.makepyfile(foo="""
+            def foo():
+              '''
+              >>> b'foo'  {comment}
+              'foo'
+              '''
+        """.format(comment=comment))
+        reprec = testdir.inline_run("--doctest-modules")
+        reprec.assertoutcome(passed=2)
+
     def test_unicode_string(self, testdir):
         """Test that doctests which output unicode fail in Python 2 when
         the ALLOW_UNICODE option is not used. The same test should pass
@@ -471,6 +549,19 @@ class TestDoctests:
         passed = int(sys.version_info[0] >= 3)
         reprec.assertoutcome(passed=passed, failed=int(not passed))
 
+    def test_bytes_literal(self, testdir):
+        """Test that doctests which output bytes fail in Python 3 when
+        the ALLOW_BYTES option is not used. The same test should pass
+        in Python 2 (#1287).
+        """
+        testdir.maketxtfile(test_doc="""
+            >>> b'foo'
+            'foo'
+        """)
+        reprec = testdir.inline_run()
+        passed = int(sys.version_info[0] == 2)
+        reprec.assertoutcome(passed=passed, failed=int(not passed))
+
 
 class TestDoctestSkips:
     """
@@ -520,6 +611,11 @@ class TestDoctestSkips:
         reprec = testdir.inline_run("--doctest-modules")
         reprec.assertoutcome(skipped=1)
 
+    def test_vacuous_all_skipped(self, testdir, makedoctest):
+        makedoctest('')
+        reprec = testdir.inline_run("--doctest-modules")
+        reprec.assertoutcome(passed=0, skipped=0)
+
 
 class TestDoctestAutoUseFixtures:
 
@@ -638,3 +734,131 @@ class TestDoctestAutoUseFixtures:
         result = testdir.runpytest('--doctest-modules')
         assert 'FAILURES' not in str(result.stdout.str())
         result.stdout.fnmatch_lines(['*=== 1 passed in *'])
+
+
+class TestDoctestNamespaceFixture:
+
+    SCOPES = ['module', 'session', 'class', 'function']
+
+    @pytest.mark.parametrize('scope', SCOPES)
+    def test_namespace_doctestfile(self, testdir, scope):
+        """
+        Check that inserting something into the namespace works in a
+        simple text file doctest
+        """
+        testdir.makeconftest("""
+            import pytest
+            import contextlib
+
+            @pytest.fixture(autouse=True, scope="{scope}")
+            def add_contextlib(doctest_namespace):
+                doctest_namespace['cl'] = contextlib
+        """.format(scope=scope))
+        p = testdir.maketxtfile("""
+            >>> print(cl.__name__)
+            contextlib
+        """)
+        reprec = testdir.inline_run(p)
+        reprec.assertoutcome(passed=1)
+
+    @pytest.mark.parametrize('scope', SCOPES)
+    def test_namespace_pyfile(self, testdir, scope):
+        """
+        Check that inserting something into the namespace works in a
+        simple Python file docstring doctest
+        """
+        testdir.makeconftest("""
+            import pytest
+            import contextlib
+
+            @pytest.fixture(autouse=True, scope="{scope}")
+            def add_contextlib(doctest_namespace):
+                doctest_namespace['cl'] = contextlib
+        """.format(scope=scope))
+        p = testdir.makepyfile("""
+            def foo():
+                '''
+                >>> print(cl.__name__)
+                contextlib
+                '''
+        """)
+        reprec = testdir.inline_run(p, "--doctest-modules")
+        reprec.assertoutcome(passed=1)
+
+
+class TestDoctestReportingOption:
+    def _run_doctest_report(self, testdir, format):
+        testdir.makepyfile("""
+            def foo():
+                '''
+                >>> foo()
+                   a  b
+                0  1  4
+                1  2  4
+                2  3  6
+                '''
+                print('   a  b\\n'
+                      '0  1  4\\n'
+                      '1  2  5\\n'
+                      '2  3  6')
+            """)
+        return testdir.runpytest("--doctest-modules", "--doctest-report", format)
+
+    @pytest.mark.parametrize('format', ['udiff', 'UDIFF', 'uDiFf'])
+    def test_doctest_report_udiff(self, testdir, format):
+        result = self._run_doctest_report(testdir, format)
+        result.stdout.fnmatch_lines([
+            '     0  1  4',
+            '    -1  2  4',
+            '    +1  2  5',
+            '     2  3  6',
+        ])
+
+    def test_doctest_report_cdiff(self, testdir):
+        result = self._run_doctest_report(testdir, 'cdiff')
+        result.stdout.fnmatch_lines([
+            '         a  b',
+            '      0  1  4',
+            '    ! 1  2  4',
+            '      2  3  6',
+            '    --- 1,4 ----',
+            '         a  b',
+            '      0  1  4',
+            '    ! 1  2  5',
+            '      2  3  6',
+        ])
+
+    def test_doctest_report_ndiff(self, testdir):
+        result = self._run_doctest_report(testdir, 'ndiff')
+        result.stdout.fnmatch_lines([
+            '         a  b',
+            '      0  1  4',
+            '    - 1  2  4',
+            '    ?       ^',
+            '    + 1  2  5',
+            '    ?       ^',
+            '      2  3  6',
+        ])
+
+    @pytest.mark.parametrize('format', ['none', 'only_first_failure'])
+    def test_doctest_report_none_or_only_first_failure(self, testdir, format):
+        result = self._run_doctest_report(testdir, format)
+        result.stdout.fnmatch_lines([
+            'Expected:',
+            '       a  b',
+            '    0  1  4',
+            '    1  2  4',
+            '    2  3  6',
+            'Got:',
+            '       a  b',
+            '    0  1  4',
+            '    1  2  5',
+            '    2  3  6',
+        ])
+
+    def test_doctest_report_invalid(self, testdir):
+        result = self._run_doctest_report(testdir, 'obviously_invalid_format')
+        result.stderr.fnmatch_lines([
+            "*error: argument --doctest-report: invalid choice: 'obviously_invalid_format' (choose from*"
+        ])
+
