@@ -1,7 +1,9 @@
-
-import py
 import sys
 
+import _pytest._code
+import pytest
+
+
 def runpdb_and_get_report(testdir, source):
     p = testdir.makepyfile(source)
     result = testdir.runpytest_inprocess("--pdb", p)
@@ -11,12 +13,14 @@ def runpdb_and_get_report(testdir, source):
 
 
 class TestPDB:
-    def pytest_funcarg__pdblist(self, request):
-        monkeypatch = request.getfuncargvalue("monkeypatch")
+
+    @pytest.fixture
+    def pdblist(self, request):
+        monkeypatch = request.getfixturevalue("monkeypatch")
         pdblist = []
         def mypdb(*args):
             pdblist.append(args)
-        plugin = request.config.pluginmanager.getplugin('pdb')
+        plugin = request.config.pluginmanager.getplugin('debugging')
         monkeypatch.setattr(plugin, 'post_mortem', mypdb)
         return pdblist
 
@@ -27,7 +31,7 @@ class TestPDB:
         """)
         assert rep.failed
         assert len(pdblist) == 1
-        tb = py.code.Traceback(pdblist[0][0])
+        tb = _pytest._code.Traceback(pdblist[0][0])
         assert tb[-1].name == "test_func"
 
     def test_pdb_on_xfail(self, testdir, pdblist):
@@ -75,6 +79,25 @@ class TestPDB:
         if child.isalive():
             child.wait()
 
+    def test_pdb_unittest_postmortem(self, testdir):
+        p1 = testdir.makepyfile("""
+            import unittest
+            class Blub(unittest.TestCase):
+                def tearDown(self):
+                    self.filename = None
+                def test_false(self):
+                    self.filename = 'debug' + '.me'
+                    assert 0
+        """)
+        child = testdir.spawn_pytest("--pdb %s" % p1)
+        child.expect('(Pdb)')
+        child.sendline('p self.filename')
+        child.sendeof()
+        rest = child.read().decode("utf8")
+        assert 'debug.me' in rest
+        if child.isalive():
+            child.wait()
+
     def test_pdb_interaction_capture(self, testdir):
         p1 = testdir.makepyfile("""
             def test_1():
@@ -291,8 +314,12 @@ class TestPDB:
 
     def test_enter_pdb_hook_is_called(self, testdir):
         testdir.makeconftest("""
-            def pytest_enter_pdb():
+            def pytest_enter_pdb(config):
+                assert config.testing_verification == 'configured'
                 print 'enter_pdb_hook'
+
+            def pytest_configure(config):
+                config.testing_verification = 'configured'
         """)
         p1 = testdir.makepyfile("""
             import pytest
@@ -306,3 +333,28 @@ class TestPDB:
         child.sendeof()
         if child.isalive():
             child.wait()
+
+    def test_pdb_custom_cls(self, testdir):
+        called = []
+
+        # install dummy debugger class and track which methods were called on it
+        class _CustomPdb:
+            def __init__(self, *args, **kwargs):
+                called.append("init")
+
+            def reset(self):
+                called.append("reset")
+
+            def interaction(self, *args):
+                called.append("interaction")
+
+        _pytest._CustomPdb = _CustomPdb
+
+        p1 = testdir.makepyfile("""xxx """)
+        result = testdir.runpytest_inprocess(
+            "--pdbcls=_pytest:_CustomPdb", p1)
+        result.stdout.fnmatch_lines([
+            "*NameError*xxx*",
+            "*1 error*",
+        ])
+        assert called == ["init", "reset", "interaction"]
