@@ -1,8 +1,12 @@
-import py, pytest
 import os
+
+import _pytest._code
+import py
+import pytest
+from _pytest.main import Node, Item, FSCollector
 from _pytest.resultlog import generic_path, ResultLog, \
         pytest_configure, pytest_unconfigure
-from _pytest.main import Node, Item, FSCollector
+
 
 def test_generic_path(testdir):
     from _pytest.main import Session
@@ -79,19 +83,10 @@ class TestWithFunctionIntegration:
 
     def test_collection_report(self, testdir):
         ok = testdir.makepyfile(test_collection_ok="")
-        skip = testdir.makepyfile(test_collection_skip=
-            "import pytest ; pytest.skip('hello')")
         fail = testdir.makepyfile(test_collection_fail="XXX")
         lines = self.getresultlog(testdir, ok)
         assert not lines
 
-        lines = self.getresultlog(testdir, skip)
-        assert len(lines) == 2
-        assert lines[0].startswith("S ")
-        assert lines[0].endswith("test_collection_skip.py")
-        assert lines[1].startswith(" ")
-        assert lines[1].endswith("test_collection_skip.py:1: Skipped: hello")
-
         lines = self.getresultlog(testdir, fail)
         assert lines
         assert lines[0].startswith("F ")
@@ -140,7 +135,7 @@ class TestWithFunctionIntegration:
         try:
             raise ValueError
         except ValueError:
-            excinfo = py.code.ExceptionInfo()
+            excinfo = _pytest._code.ExceptionInfo()
         reslog = ResultLog(None, py.io.TextIO())
         reslog.pytest_internalerror(excinfo.getrepr(style=style))
         entry = reslog.logfile.getvalue()
@@ -227,6 +222,6 @@ def test_failure_issue380(testdir):
             pass
     """)
     result = testdir.runpytest("--resultlog=log")
-    assert result.ret == 1
+    assert result.ret == 2
 
 
