@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from __future__ import with_statement
 
 import _pytest._code
@@ -227,6 +228,40 @@ class BaseFunctionalTests:
         assert reps[5].nodeid.endswith("test_func")
         assert reps[5].failed
 
+    def test_exact_teardown_issue1206(self, testdir):
+        """issue shadowing error with wrong number of arguments on teardown_method."""
+        rec = testdir.inline_runsource("""
+            import pytest
+
+            class TestClass:
+                def teardown_method(self, x, y, z):
+                    pass
+
+                def test_method(self):
+                    assert True
+        """)
+        reps = rec.getreports("pytest_runtest_logreport")
+        print (reps)
+        assert len(reps) == 3
+        #
+        assert reps[0].nodeid.endswith("test_method")
+        assert reps[0].passed
+        assert reps[0].when == 'setup'
+        #
+        assert reps[1].nodeid.endswith("test_method")
+        assert reps[1].passed
+        assert reps[1].when == 'call'
+        #
+        assert reps[2].nodeid.endswith("test_method")
+        assert reps[2].failed
+        assert reps[2].when == "teardown"
+        assert reps[2].longrepr.reprcrash.message in (
+                # python3 error
+                "TypeError: teardown_method() missing 2 required positional arguments: 'y' and 'z'",
+                # python2 error
+                'TypeError: teardown_method() takes exactly 4 arguments (2 given)'
+                )
+
     def test_failure_in_setup_function_ignores_custom_repr(self, testdir):
         testdir.makepyfile(conftest="""
             import pytest
@@ -331,18 +366,6 @@ class TestSessionReports:
         assert res[0].name == "test_func1"
         assert res[1].name == "TestClass"
 
-    def test_skip_at_module_scope(self, testdir):
-        col = testdir.getmodulecol("""
-            import pytest
-            pytest.skip("hello")
-            def test_func():
-                pass
-        """)
-        rep = main.collect_one_node(col)
-        assert not rep.failed
-        assert not rep.passed
-        assert rep.skipped
-
 
 reporttypes = [
     runner.BaseReport,
@@ -377,13 +400,15 @@ def test_callinfo():
 @pytest.mark.xfail
 def test_runtest_in_module_ordering(testdir):
     p1 = testdir.makepyfile("""
+        import pytest
         def pytest_runtest_setup(item): # runs after class-level!
             item.function.mylist.append("module")
         class TestClass:
             def pytest_runtest_setup(self, item):
                 assert not hasattr(item.function, 'mylist')
                 item.function.mylist = ['class']
-            def pytest_funcarg__mylist(self, request):
+            @pytest.fixture
+            def mylist(self, request):
                 return request.function.mylist
             def pytest_runtest_call(self, item, __multicall__):
                 try:
@@ -423,6 +448,18 @@ def test_pytest_fail():
         s = excinfo.exconly(tryshort=True)
         assert s.startswith("Failed")
 
+def test_pytest_exit_msg(testdir):
+    testdir.makeconftest("""
+    import pytest
+
+    def pytest_configure(config):
+        pytest.exit('oh noes')
+    """)
+    result = testdir.runpytest()
+    result.stderr.fnmatch_lines([
+        "Exit: oh noes",
+    ])
+
 def test_pytest_fail_notrace(testdir):
     testdir.makepyfile("""
         import pytest
@@ -439,6 +476,27 @@ def test_pytest_fail_notrace(testdir):
     assert 'def teardown_function' not in result.stdout.str()
 
 
+@pytest.mark.parametrize('str_prefix', ['u', ''])
+def test_pytest_fail_notrace_non_ascii(testdir, str_prefix):
+    """Fix pytest.fail with pytrace=False with non-ascii characters (#1178).
+
+    This tests with native and unicode strings containing non-ascii chars.
+    """
+    testdir.makepyfile(u"""
+        # coding: utf-8
+        import pytest
+
+        def test_hello():
+            pytest.fail(%s'oh oh: ☺', pytrace=False)
+    """ % str_prefix)
+    result = testdir.runpytest()
+    if sys.version_info[0] >= 3:
+        result.stdout.fnmatch_lines(['*test_hello*', "oh oh: ☺"])
+    else:
+        result.stdout.fnmatch_lines(['*test_hello*', "oh oh: *"])
+    assert 'def test_hello' not in result.stdout.str()
+
+
 def test_pytest_no_tests_collected_exit_status(testdir):
     result = testdir.runpytest()
     result.stdout.fnmatch_lines('*collected 0 items*')
@@ -513,6 +571,19 @@ def test_importorskip_dev_module(monkeypatch):
         pytest.fail("spurious skip")
 
 
+def test_importorskip_module_level(testdir):
+    """importorskip must be able to skip entire modules when used at module level"""
+    testdir.makepyfile('''
+        import pytest
+        foobarbaz = pytest.importorskip("foobarbaz")
+
+        def test_foo():
+            pass
+    ''')
+    result = testdir.runpytest()
+    result.stdout.fnmatch_lines(['*collected 0 items / 1 skipped*'])
+
+
 def test_pytest_cmdline_main(testdir):
     p = testdir.makepyfile("""
         import pytest
@@ -610,3 +681,68 @@ def test_store_except_info_on_eror():
     assert sys.last_type is IndexError
     assert sys.last_value.args[0] == 'TEST'
     assert sys.last_traceback
+
+
+class TestReportContents:
+    """
+    Test user-level API of ``TestReport`` objects.
+    """
+
+    def getrunner(self):
+        return lambda item: runner.runtestprotocol(item, log=False)
+
+    def test_longreprtext_pass(self, testdir):
+        reports = testdir.runitem("""
+            def test_func():
+                pass
+        """)
+        rep = reports[1]
+        assert rep.longreprtext == ''
+
+    def test_longreprtext_failure(self, testdir):
+        reports = testdir.runitem("""
+            def test_func():
+                x = 1
+                assert x == 4
+        """)
+        rep = reports[1]
+        assert 'assert 1 == 4' in rep.longreprtext
+
+    def test_captured_text(self, testdir):
+        reports = testdir.runitem("""
+            import pytest
+            import sys
+
+            @pytest.fixture
+            def fix():
+                sys.stdout.write('setup: stdout\\n')
+                sys.stderr.write('setup: stderr\\n')
+                yield
+                sys.stdout.write('teardown: stdout\\n')
+                sys.stderr.write('teardown: stderr\\n')
+                assert 0
+
+            def test_func(fix):
+                sys.stdout.write('call: stdout\\n')
+                sys.stderr.write('call: stderr\\n')
+                assert 0
+        """)
+        setup, call, teardown = reports
+        assert setup.capstdout == 'setup: stdout\n'
+        assert call.capstdout == 'setup: stdout\ncall: stdout\n'
+        assert teardown.capstdout == 'setup: stdout\ncall: stdout\nteardown: stdout\n'
+
+        assert setup.capstderr == 'setup: stderr\n'
+        assert call.capstderr == 'setup: stderr\ncall: stderr\n'
+        assert teardown.capstderr == 'setup: stderr\ncall: stderr\nteardown: stderr\n'
+
+    def test_no_captured_text(self, testdir):
+        reports = testdir.runitem("""
+            def test_func():
+                pass
+        """)
+        rep = reports[1]
+        assert rep.capstdout == ''
+        assert rep.capstderr == ''
+
+
